@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CMaintenance extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN, 'action' => CRoleHelper::ACTIONS_EDIT_MAINTENANCE],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN, 'action' => CRoleHelper::ACTIONS_EDIT_MAINTENANCE],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN, 'action' => CRoleHelper::ACTIONS_EDIT_MAINTENANCE]
+	];
+
 	protected $tableName = 'maintenances';
 	protected $tableAlias = 'm';
 	protected $sortColumns = ['maintenanceid', 'name', 'maintenance_type', 'active_till', 'active_since'];
@@ -70,12 +77,12 @@ class CMaintenance extends CApiService {
 			'searchByAny'				=> null,
 			'startSearch'				=> false,
 			'excludeSearch'				=> false,
-			'filter'					=> null,
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectGroups'				=> null,
 			'selectHosts'				=> null,
+			'selectTags'				=> null,
 			'selectTimeperiods'			=> null,
 			'countOutput'				=> false,
 			'groupCount'				=> false,
@@ -219,7 +226,7 @@ class CMaintenance extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($maintenance = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -253,7 +260,9 @@ class CMaintenance extends CApiService {
 	 *
 	 * @param array $maintenances
 	 *
-	 * @return boolean
+	 * @throws APIException if no permissions to object, it does no exists or validation errors.
+	 *
+	 * @return array
 	 */
 	public function create(array $maintenances) {
 		$maintenances = zbx_toArray($maintenances);
@@ -264,13 +273,17 @@ class CMaintenance extends CApiService {
 		$hostids = [];
 		$groupids = [];
 		foreach ($maintenances as $maintenance) {
-			$hostids = array_merge($hostids, $maintenance['hostids']);
-			$groupids = array_merge($groupids, $maintenance['groupids']);
+			if (array_key_exists('hostids', $maintenance)) {
+				$hostids = array_merge($hostids, $maintenance['hostids']);
+			}
+			if (array_key_exists('groupids', $maintenance)) {
+				$groupids = array_merge($groupids, $maintenance['groupids']);
+			}
 		}
 
 		// validate hosts & groups
 		if (empty($hostids) && empty($groupids)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one host or group should be selected.'));
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one host group or host must be selected.'));
 		}
 
 		// hosts permissions
@@ -300,8 +313,6 @@ class CMaintenance extends CApiService {
 			}
 		}
 
-		$this->removeSecondsFromTimes($maintenances);
-
 		$tid = 0;
 		$insert = [];
 		$timeperiods = [];
@@ -344,17 +355,23 @@ class CMaintenance extends CApiService {
 		foreach ($maintenances as $mnum => $maintenance) {
 			// validate maintenance active since
 			if (!validateUnixTime($maintenance['active_since'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active since')));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'active_since')
+				);
 			}
 
 			// validate maintenance active till
 			if (!validateUnixTime($maintenance['active_till'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active till')));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'active_till')
+				);
 			}
 
 			// validate maintenance active interval
 			if ($maintenance['active_since'] > $maintenance['active_till']) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Maintenance "Active since" value cannot be bigger than "Active till".'));
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Maintenance "%1$s" value cannot be bigger than "%2$s".', 'active_since', 'active_till')
+				);
 			}
 
 			// validate timeperiods
@@ -363,6 +380,9 @@ class CMaintenance extends CApiService {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one maintenance period must be created.'));
 			}
 
+			$maintenance['active_since'] -= $maintenance['active_since'] % SEC_PER_MIN;
+			$maintenance['active_till'] -= $maintenance['active_till'] % SEC_PER_MIN;
+
 			foreach ($maintenance['timeperiods'] as $timeperiod) {
 				if (!is_array($timeperiod)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one maintenance period must be created.'));
@@ -375,13 +395,34 @@ class CMaintenance extends CApiService {
 				];
 				check_db_fields($dbFields, $timeperiod);
 
+				if (array_key_exists('every', $timeperiod) && $timeperiod['every'] <= 0) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect value "%1$s" for unsigned int field "%2$s".', $timeperiod['every'], 'every')
+					);
+				}
+
+				if ($timeperiod['timeperiod_type'] != TIMEPERIOD_TYPE_ONETIME) {
+					$timeperiod['start_date'] = DB::getDefault('timeperiods', 'start_date');
+				}
+				else if (!validateUnixTime($timeperiod['start_date'])) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'start_date')
+					);
+				}
+				else {
+					$timeperiod['start_date'] -= $timeperiod['start_date'] % SEC_PER_MIN;
+				}
+
 				$tid++;
 				$insertTimeperiods[$tid] = $timeperiod;
 				$timeperiods[$tid] = $mnum;
 			}
 
 			$insert[$mnum] = $maintenance;
+
+			$this->validateTags($maintenance);
 		}
+
 		$maintenanceids = DB::insert('maintenances', $insert);
 		$timeperiodids = DB::insert('timeperiods', $insertTimeperiods);
 
@@ -396,30 +437,88 @@ class CMaintenance extends CApiService {
 
 		$insertHosts = [];
 		$insertGroups = [];
+		$ins_tags = [];
 		foreach ($maintenances as $mnum => $maintenance) {
-			foreach ($maintenance['hostids'] as $hostid) {
-				$insertHosts[] = [
-					'hostid' => $hostid,
-					'maintenanceid' => $maintenanceids[$mnum]
-				];
+			if (array_key_exists('hostids', $maintenance)) {
+				foreach ($maintenance['hostids'] as $hostid) {
+					$insertHosts[] = [
+						'hostid' => $hostid,
+						'maintenanceid' => $maintenanceids[$mnum]
+					];
+				}
 			}
-			foreach ($maintenance['groupids'] as $groupid) {
-				$insertGroups[] = [
-					'groupid' => $groupid,
-					'maintenanceid' => $maintenanceids[$mnum]
-				];
+
+			if (array_key_exists('groupids', $maintenance)) {
+				foreach ($maintenance['groupids'] as $groupid) {
+					$insertGroups[] = [
+						'groupid' => $groupid,
+						'maintenanceid' => $maintenanceids[$mnum]
+					];
+				}
+			}
+
+			if (array_key_exists('tags', $maintenance)) {
+				foreach ($maintenance['tags'] as $tag) {
+					$ins_tags[] = [
+						'maintenanceid' => $maintenanceids[$mnum]
+					] + $tag;
+				}
 			}
 
 			add_audit_details(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MAINTENANCE, $maintenanceids[$mnum],
-				$maintenance['name'], null
+				$maintenance['name'], null, self::$userData['userid']
 			);
 		}
 		DB::insert('maintenances_hosts', $insertHosts);
 		DB::insert('maintenances_groups', $insertGroups);
 
+		if ($ins_tags) {
+			DB::insert('maintenance_tag', $ins_tags);
+		}
+
 		return ['maintenanceids' => $maintenanceids];
 	}
 
+	/**
+	 * Validates maintenance problem tags.
+	 *
+	 * @param array  $maintenance
+	 * @param int    $maintenance['maintenance_type']
+	 * @param int    $maintenance['tags_evaltype']
+	 * @param array  $maintenance['tags']
+	 * @param string $maintenance['tags'][]['tag']
+	 * @param int    $maintenance['tags'][]['operator']
+	 * @param string $maintenance['tags'][]['value']
+	 *
+	 * @throws APIException if the input is invalid.
+	 */
+	private function validateTags(array $maintenance) {
+		if (array_key_exists('maintenance_type', $maintenance)
+				&& $maintenance['maintenance_type'] == MAINTENANCE_TYPE_NODATA
+				&& array_key_exists('tags', $maintenance) && $maintenance['tags']) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Incorrect value for field "%1$s": %2$s.', 'tags', _('should be empty'))
+			);
+		}
+
+		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+			'maintenance_type'	=> ['type' => API_INT32, 'in' => implode(',', [MAINTENANCE_TYPE_NORMAL, MAINTENANCE_TYPE_NODATA])],
+			'tags_evaltype'		=> ['type' => API_INT32, 'in' => implode(',', [MAINTENANCE_TAG_EVAL_TYPE_AND_OR, MAINTENANCE_TAG_EVAL_TYPE_OR])],
+			'tags'				=> ['type' => API_OBJECTS, 'uniq' => [['tag', 'operator', 'value']], 'fields' => [
+				'tag'				=> ['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('maintenance_tag', 'tag')],
+				'operator'			=> ['type' => API_INT32, 'in' => implode(',', [MAINTENANCE_TAG_OPERATOR_EQUAL, MAINTENANCE_TAG_OPERATOR_LIKE]), 'default' => DB::getDefault('maintenance_tag', 'operator')],
+				'value'				=> ['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('maintenance_tag', 'value'), 'default' => DB::getDefault('maintenance_tag', 'value')]
+			]]
+		]];
+
+		// Keep values only for fields with defined validation rules.
+		$maintenance = array_intersect_key($maintenance, $api_input_rules['fields']);
+
+		if (!CApiInputValidator::validate($api_input_rules, $maintenance, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+	}
+
 	/**
 	 * Update maintenances.
 	 *
@@ -450,6 +549,8 @@ class CMaintenance extends CApiService {
 			if (!check_db_fields($db_fields, $maintenance)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect parameters for maintenance.'));
 			}
+
+			$this->validateTags($maintenance);
 		}
 
 		$db_maintenances = $this->get([
@@ -466,16 +567,18 @@ class CMaintenance extends CApiService {
 		$hostids = [];
 		$groupids = [];
 
-		foreach ($maintenances as $maintenance) {
+		foreach ($maintenances as &$maintenance) {
 			if (!array_key_exists($maintenance['maintenanceid'], $db_maintenances)) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS,
 					_('No permissions to referred object or it does not exist!')
 				);
 			}
 
+			$db_maintenance = $db_maintenances[$maintenance['maintenanceid']];
+
 			// Check maintenances names and collect for unique checking.
 			if (array_key_exists('name', $maintenance) && $maintenance['name'] !== ''
-					&& $db_maintenances[$maintenance['maintenanceid']]['name'] !== $maintenance['name']) {
+					&& $db_maintenance['name'] !== $maintenance['name']) {
 				if (array_key_exists($maintenance['name'], $changed_names)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
 						_s('Maintenance "%1$s" already exists.', $maintenance['name'])
@@ -491,12 +594,14 @@ class CMaintenance extends CApiService {
 
 				if (!validateUnixTime($active_since)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active since'))
+						_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'active_since')
 					);
 				}
+
+				$maintenance['active_since'] -= $maintenance['active_since'] % SEC_PER_MIN;
 			}
 			else {
-				$active_since = $db_maintenances[$maintenance['maintenanceid']]['active_since'];
+				$active_since = $db_maintenance['active_since'];
 			}
 
 			// Validate maintenance active till.
@@ -505,12 +610,14 @@ class CMaintenance extends CApiService {
 
 				if (!validateUnixTime($active_till)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('"%s" must be between 1970.01.01 and 2038.01.18.', _('Active till'))
+						_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'active_till')
 					);
 				}
+
+				$maintenance['active_till'] -= $maintenance['active_till'] % SEC_PER_MIN;
 			}
 			else {
-				$active_till = $db_maintenances[$maintenance['maintenanceid']]['active_till'];
+				$active_till = $db_maintenance['active_till'];
 			}
 
 			// Validate maintenance active interval.
@@ -526,13 +633,60 @@ class CMaintenance extends CApiService {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one maintenance period must be created.'));
 				}
 
-				foreach ($maintenance['timeperiods'] as $timeperiod) {
+				$db_timeperiods = zbx_toHash($db_maintenance['timeperiods'], 'timeperiodid');
+
+				foreach ($maintenance['timeperiods'] as &$timeperiod) {
 					if (!is_array($timeperiod)) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_('At least one maintenance period must be created.')
 						);
 					}
+
+					$timeperiod_type = array_key_exists('timeperiod_type', $timeperiod)
+						? $timeperiod['timeperiod_type']
+						: null;
+
+					if (array_key_exists('timeperiodid', $timeperiod)) {
+						$timeperiodid = $timeperiod['timeperiodid'];
+
+						// Validate incorrect "timeperiodid".
+						if (!array_key_exists($timeperiodid, $db_timeperiods)) {
+							self::exception(ZBX_API_ERROR_PERMISSIONS,
+								_('No permissions to referred object or it does not exist!')
+							);
+						}
+
+						if ($timeperiod_type === null) {
+							$timeperiod_type = $db_timeperiods[$timeperiodid]['timeperiod_type'];
+						}
+					}
+
+					if (array_key_exists('every', $timeperiod) && $timeperiod['every'] <= 0) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Incorrect value "%1$s" for unsigned int field "%2$s".', $timeperiod['every'], 'every')
+						);
+					}
+
+					// Without "timeperiod_type" it resolves to default TIMEPERIOD_TYPE_ONETIME. But will it be forever?
+					if ($timeperiod_type === null) {
+						$timeperiod_type = DB::getDefault('timeperiods', 'timeperiod_type');
+					}
+
+					// Reset "start_date" to default value in case "timeperiod_type" is not one time only.
+					if ($timeperiod_type != TIMEPERIOD_TYPE_ONETIME) {
+						$timeperiod['start_date'] = DB::getDefault('timeperiods', 'start_date');
+					}
+					else if (array_key_exists('start_date', $timeperiod)
+							&& !validateUnixTime($timeperiod['start_date'])) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('"%1$s" must be between 1970.01.01 and 2038.01.18.', 'start_date')
+						);
+					}
+					else {
+						$timeperiod['start_date'] -= $timeperiod['start_date'] % SEC_PER_MIN;
+					}
 				}
+				unset($timeperiod);
 			}
 
 			// Collect hostids for permission checking.
@@ -554,9 +708,22 @@ class CMaintenance extends CApiService {
 			}
 
 			if (!$has_hosts && !$has_groups) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one host or group should be selected.'));
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('At least one host group or host must be selected.'));
+			}
+
+			// Check if maintenance without data collection has no tags.
+			$db_maintenance_type = $db_maintenances[$maintenance['maintenanceid']]['maintenance_type'];
+			$maintenance_type = array_key_exists('maintenance_type', $maintenance)
+				? $maintenance['maintenance_type']
+				: $db_maintenance_type;
+			if ($db_maintenance_type == MAINTENANCE_TYPE_NODATA && $maintenance_type == $db_maintenance_type
+					&& array_key_exists('tags', $maintenance) && $maintenance['tags']) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect value for field "%1$s": %2$s.', 'tags', _('should be empty'))
+				);
 			}
 		}
+		unset($maintenance);
 
 		// Check if maintenance already exists.
 		if ($changed_names) {
@@ -611,8 +778,6 @@ class CMaintenance extends CApiService {
 			}
 		}
 
-		$this->removeSecondsFromTimes($maintenances);
-
 		$update_maintenances = [];
 		foreach ($maintenances as $mnum => $maintenance) {
 			$update_maintenances[$mnum] = [
@@ -700,9 +865,93 @@ class CMaintenance extends CApiService {
 			DB::insert('maintenances_groups', $insert_groups);
 		}
 
+		$this->updateTags($maintenances, $db_maintenances);
+
 		return ['maintenanceids' => $maintenanceids];
 	}
 
+	/**
+	 * Compares input tags with tags stored in the database and performs tag deleting and inserting.
+	 *
+	 * @param array  $maintenances
+	 * @param int    $maintenances[]['maintenanceid']
+	 * @param int    $maintenances[]['maintenance_type']
+	 * @param array  $maintenances[]['tags']
+	 * @param string $maintenances[]['tags'][]['tag']
+	 * @param int    $maintenances[]['tags'][]['operator']
+	 * @param string $maintenances[]['tags'][]['value']
+	 * @param array  $db_maintenances
+	 * @param int    $db_maintenances[<maintenanceid>]
+	 * @param int    $db_maintenances[<maintenanceid>]['maintenance_type']
+	 */
+	private function updateTags(array $maintenances, array $db_maintenances) {
+		$db_tags = API::getApiService()->select('maintenance_tag', [
+			'output' => ['maintenancetagid', 'maintenanceid', 'tag', 'operator', 'value'],
+			'filter' => ['maintenanceid' => array_keys($db_maintenances)],
+			'preservekeys' => true
+		]);
+		$relation_map = $this->createRelationMap($db_tags, 'maintenanceid', 'maintenancetagid');
+		$db_maintenances = $relation_map->mapMany($db_maintenances, $db_tags, 'tags');
+
+		$ins_tags = [];
+		$del_maintenancetagids = [];
+
+		foreach ($maintenances as $mnum => $maintenance) {
+			$maintenanceid = $maintenance['maintenanceid'];
+
+			if (array_key_exists('maintenance_type', $maintenance)
+					&& $maintenance['maintenance_type'] == MAINTENANCE_TYPE_NODATA
+					&& $db_maintenances[$maintenanceid]['tags']) {
+				foreach ($db_maintenances[$maintenanceid]['tags'] as $db_tag) {
+					$del_maintenancetagids[] = $db_tag['maintenancetagid'];
+				}
+				unset($maintenances[$mnum], $db_maintenances[$maintenanceid]);
+				continue;
+			}
+
+			if (!array_key_exists('tags', $maintenance)) {
+				unset($maintenances[$mnum], $db_maintenances[$maintenanceid]);
+				continue;
+			}
+
+			foreach ($maintenance['tags'] as $tag_num => $tag) {
+				$tag += [
+					'operator' => MAINTENANCE_TAG_OPERATOR_LIKE,
+					'value' => ''
+				];
+
+				foreach ($db_maintenances[$maintenanceid]['tags'] as $db_tag_num => $db_tag) {
+					if ($tag['tag'] === $db_tag['tag'] && $tag['operator'] == $db_tag['operator']
+							&& $tag['value'] === $db_tag['value']) {
+						unset($maintenances[$mnum]['tags'][$tag_num],
+							$db_maintenances[$maintenanceid]['tags'][$db_tag_num]
+						);
+					}
+				}
+			}
+		}
+
+		foreach ($maintenances as $maintenance) {
+			$maintenanceid = $maintenance['maintenanceid'];
+
+			foreach ($maintenance['tags'] as $tag) {
+				$ins_tags[] = ['maintenanceid' => $maintenanceid] + $tag;
+			}
+
+			foreach ($db_maintenances[$maintenanceid]['tags'] as $db_tag) {
+				$del_maintenancetagids[] = $db_tag['maintenancetagid'];
+			}
+		}
+
+		if ($del_maintenancetagids) {
+			DB::delete('maintenance_tag', ['maintenancetagid' => $del_maintenancetagids]);
+		}
+
+		if ($ins_tags) {
+			DB::insert('maintenance_tag', $ins_tags);
+		}
+	}
+
 	/**
 	 * Delete Maintenances.
 	 *
@@ -715,15 +964,15 @@ class CMaintenance extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
 		}
 
-		$options = [
+		$maintenances = $this->get([
 			'output' => ['maintenanceid', 'name'],
 			'maintenanceids' => $maintenanceids,
 			'editable' => true,
 			'preservekeys' => true
-		];
-		$maintenances = $this->get($options);
+		]);
+
 		foreach ($maintenanceids as $maintenanceid) {
-			if (!isset($maintenances[$maintenanceid])) {
+			if (!array_key_exists($maintenanceid, $maintenances)) {
 				self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
 			}
 		}
@@ -741,19 +990,19 @@ class CMaintenance extends CApiService {
 
 		$midCond = ['maintenanceid' => $maintenanceids];
 
-		// remove maintenanceid from hosts table
-		$options = [
-			'real_hosts' => true,
-			'output' => ['hostid'],
-			'filter' => ['maintenanceid' => $maintenanceids]
-		];
-		$hosts = API::Host()->get($options);
-		if (!empty($hosts)) {
-			DB::update('hosts', [
-				'values' => ['maintenanceid' => 0],
-				'where' => ['hostid' => zbx_objectValues($hosts, 'hostid')]
-			]);
-		}
+		// Lock maintenances table before maintenance delete to prevent server from adding host to maintenance.
+		DBselect(
+			'SELECT NULL'.
+			' FROM maintenances'.
+			' WHERE '.dbConditionId('maintenanceid', $maintenanceids).
+			' FOR UPDATE'
+		);
+
+		// Remove maintenanceid from hosts table.
+		DB::update('hosts', [
+			'values' => ['maintenanceid' => 0],
+			'where' => ['maintenanceid' => $maintenanceids]
+		]);
 
 		DB::delete('timeperiods', ['timeperiodid' => $timeperiodids]);
 		DB::delete('maintenances_windows', $midCond);
@@ -763,41 +1012,13 @@ class CMaintenance extends CApiService {
 
 		foreach ($maintenances as $maintenanceid => $maintenance) {
 			add_audit_details(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_MAINTENANCE, $maintenanceid, $maintenance['name'],
-				null
+				null, self::$userData['userid']
 			);
 		}
 
 		return ['maintenanceids' => $maintenanceids];
 	}
 
-	/**
-	 * Reset seconds to zero in maintenace time values.
-	 *
-	 * @param array $maintenances passed by reference
-	 */
-	protected function removeSecondsFromTimes(array &$maintenances) {
-		foreach ($maintenances as &$maintenance) {
-			if (isset($maintenance['active_since'])) {
-				$maintenance['active_since'] -= $maintenance['active_since'] % SEC_PER_MIN;
-			}
-
-			if (isset($maintenance['active_till'])) {
-				$maintenance['active_till'] -= $maintenance['active_till'] % SEC_PER_MIN;
-			}
-
-
-			if (isset($maintenance['timeperiods'])) {
-				foreach ($maintenance['timeperiods'] as &$timeperiod) {
-					if (isset($timeperiod['start_date'])) {
-						$timeperiod['start_date'] -= $timeperiod['start_date'] % SEC_PER_MIN;
-					}
-				}
-				unset($timeperiod);
-			}
-		}
-		unset($maintenance);
-	}
-
 	/**
 	 * Updates maintenance time periods.
 	 *
@@ -847,6 +1068,18 @@ class CMaintenance extends CApiService {
 			$result = $relationMap->mapMany($result, $groups, 'hosts');
 		}
 
+		// Adding problem tags.
+		if ($options['selectTags'] !== null && $options['selectTags'] != API_OUTPUT_COUNT) {
+			$tags = API::getApiService()->select('maintenance_tag', [
+				'output' => $this->outputExtend($options['selectTags'], ['maintenanceid']),
+				'filter' => ['maintenanceids' => array_keys($result)],
+				'preservekeys' => true
+			]);
+			$relation_map = $this->createRelationMap($tags, 'maintenanceid', 'maintenancetagid');
+			$tags = $this->unsetExtraFields($tags, ['maintenancetagid', 'maintenanceid'], []);
+			$result = $relation_map->mapMany($result, $tags, 'tags');
+		}
+
 		// selectTimeperiods
 		if ($options['selectTimeperiods'] !== null && $options['selectTimeperiods'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'maintenanceid', 'timeperiodid', 'maintenances_windows');
