@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -18,8 +18,9 @@
 ** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 **/
 
+
 class CTriggerExpression {
-	// for parsing of trigger expression
+	// For parsing of trigger expression.
 	const STATE_AFTER_OPEN_BRACE = 1;
 	const STATE_AFTER_BINARY_OPERATOR = 2;
 	const STATE_AFTER_LOGICAL_OPERATOR = 3;
@@ -28,6 +29,11 @@ class CTriggerExpression {
 	const STATE_AFTER_CLOSE_BRACE = 6;
 	const STATE_AFTER_CONSTANT = 7;
 
+	// Error type constants.
+	const ERROR_LEVEL = 1;
+	const ERROR_UNEXPECTED_ENDING = 2;
+	const ERROR_UNPARSED_CONTENT = 3;
+
 	/**
 	 * Shows a validity of trigger expression
 	 *
@@ -42,6 +48,20 @@ class CTriggerExpression {
 	 */
 	public $error;
 
+	/**
+	 * Type of parsing error, on of self::ERROR_* constant or 0 when no errors.
+	 *
+	 * @var int
+	 */
+	public $error_type;
+
+	/**
+	 * In case of error contain failed position in expression string. Contain -1 when no errors.
+	 *
+	 * @var int
+	 */
+	public $error_pos;
+
 	/**
 	 * An array of trigger functions like {Zabbix server:agent.ping.last(0)}
 	 * The array isn't unique. Same functions can repeat.
@@ -53,14 +73,25 @@ class CTriggerExpression {
 	public $expressions = [];
 
 	/**
-	 * An options array
+	 * An options array.
 	 *
 	 * Supported options:
-	 *   'lldmacros' => true	Enable low-level discovery macros usage in trigger expression.
+	 *   'lldmacros' => true             Enable low-level discovery macros usage in trigger expression.
+	 *   'allow_func_only' => true       Allow trigger expression without host:key pair, i.e. {func(param)}.
+	 *   'collapsed_expression' => true  Short trigger expression.
+	 *                                       For example: {439} > {$MAX_THRESHOLD} or {439} < {$MIN_THRESHOLD}
+	 *   'calculated' => false           Parse calculated item formula instead of trigger expression.
+	 *   'host_macro'                    Array of macro supported as host name part in function.
 	 *
 	 * @var array
 	 */
-	public $options = ['lldmacros' => true];
+	public $options = [
+		'lldmacros' => true,
+		'allow_func_only' => false,
+		'collapsed_expression' => false,
+		'calculated' => false,
+		'host_macro' => []
+	];
 
 	/**
 	 * Source string.
@@ -69,6 +100,13 @@ class CTriggerExpression {
 	 */
 	public $expression;
 
+	/**
+	 * Object containing the results of parsing.
+	 *
+	 * @var CTriggerExprParserResult
+	 */
+	public $result;
+
 	/**
 	 * Current cursor position.
 	 *
@@ -111,6 +149,13 @@ class CTriggerExpression {
 	 */
 	protected $function_macro_parser;
 
+	/**
+	 * Parser for function id macros.
+	 *
+	 * @var CFunctionIdParser
+	 */
+	protected $functionid_parser;
+
 	/**
 	 * Parser for trigger functions.
 	 *
@@ -139,6 +184,13 @@ class CTriggerExpression {
 	 */
 	protected $user_macro_parser;
 
+	/**
+	 * Parser for numbers with optional time or byte suffix.
+	 *
+	 * @var CNumberParser
+	 */
+	protected $number_parser;
+
 	/**
 	 * Chars that should be treated as spaces.
 	 *
@@ -146,31 +198,32 @@ class CTriggerExpression {
 	 */
 	protected $spaceChars = [' ' => true, "\r" => true, "\n" => true, "\t" => true];
 
-	/**
-	 * Object containing the results of parsing.
-	 *
-	 * @var CTriggerExpressionParserResult
-	 */
-	protected $result;
-
 	/**
 	 * @param array $options
-	 * @param bool $options['lldmacros']
+	 * @param bool  $options['lldmacros']
+	 * @param bool  $options['allow_func_only']
+	 * @param bool  $options['collapsed_expression']
+	 * @param bool  $options['calculated']
+	 * @param bool  $options['host_macro']
 	 */
-	public function __construct($options = []) {
-		if (isset($options['lldmacros'])) {
-			$this->options['lldmacros'] = $options['lldmacros'];
-		}
+	public function __construct(array $options = []) {
+		$this->options = $options + $this->options;
 
 		$this->binaryOperatorParser = new CSetParser(['<', '>', '<=', '>=', '+', '-', '/', '*', '=', '<>']);
 		$this->logicalOperatorParser = new CSetParser(['and', 'or']);
 		$this->notOperatorParser = new CSetParser(['not']);
-		$this->macro_parser = new CMacroParser(['{TRIGGER.VALUE}']);
-		$this->function_macro_parser = new CFunctionMacroParser();
+		$this->macro_parser = new CMacroParser(['macros' => ['{TRIGGER.VALUE}']]);
+		if ($this->options['collapsed_expression']) {
+			$this->functionid_parser = new CFunctionIdParser();
+		}
+		else {
+			$this->function_macro_parser = new CFunctionMacroParser(['host_macro' => $this->options['host_macro']]);
+		}
 		$this->function_parser = new CFunctionParser();
 		$this->lld_macro_parser = new CLLDMacroParser();
 		$this->lld_macro_function_parser = new CLLDMacroFunctionParser;
 		$this->user_macro_parser = new CUserMacroParser();
+		$this->number_parser = new CNumberParser(['with_minus' => false, 'with_suffix' => true]);
 	}
 
 	/**
@@ -198,18 +251,25 @@ class CTriggerExpression {
 	 *
 	 * @param string $expression
 	 *
-	 * @return CTriggerExpressionParserResult|bool   returns a result object if a match has been found or false otherwise
+	 * @return CTriggerExprParserResult|bool   returns a result object if a match has been found or false otherwise
 	 */
 	public function parse($expression) {
 		// initializing local variables
-		$this->result = new CTriggerExpressionParserResult();
+		$this->result = new CTriggerExprParserResult();
 		$this->isValid = true;
 		$this->error = '';
+		$this->error_type = 0;
+		$this->error_pos = -1;
 		$this->expressions = [];
 
 		$this->pos = 0;
 		$this->expression = $expression;
 
+		if ($this->options['collapsed_expression'] && $this->options['allow_func_only']) {
+			$this->isValid = false;
+			$this->error = 'Incompatible options.';
+		}
+
 		$state = self::STATE_AFTER_OPEN_BRACE;
 		$afterSpace = false;
 		$level = 0;
@@ -228,20 +288,22 @@ class CTriggerExpression {
 					switch ($char) {
 						case '-':
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							break;
+
 						case '(':
 							$state = self::STATE_AFTER_OPEN_BRACE;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$level++;
 							break;
+
 						default:
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
 							elseif ($this->parseConstant()) {
@@ -251,23 +313,25 @@ class CTriggerExpression {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_BINARY_OPERATOR:
 					switch ($char) {
 						case '-':
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							break;
+
 						case '(':
 							$state = self::STATE_AFTER_OPEN_BRACE;
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$level++;
 							break;
+
 						default:
 							if ($this->parseConstant()) {
 								$state = self::STATE_AFTER_CONSTANT;
@@ -279,42 +343,42 @@ class CTriggerExpression {
 							}
 
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
 							else {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_LOGICAL_OPERATOR:
 					switch ($char) {
 						case '-':
 							if (!$afterSpace) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
 							break;
+
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
 							$level++;
 							break;
+
 						default:
 							if (!$afterSpace) {
 								break 3;
 							}
 
 							if ($this->parseUsing($this->notOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_NOT_OPERATOR;
 							}
 							elseif ($this->parseConstant()) {
@@ -324,30 +388,29 @@ class CTriggerExpression {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_CLOSE_BRACE:
 					switch ($char) {
 						case ')':
 							if ($level == 0) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_CLOSE_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_CLOSE_BRACE,
 								$char, $this->pos, 1
 							);
 							$level--;
 							break;
+
 						default:
 							if ($this->parseUsing($this->binaryOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_BINARY_OPERATOR;
 								break;
 							}
 
 							if ($this->parseUsing($this->logicalOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
 								break;
 							}
@@ -355,24 +418,24 @@ class CTriggerExpression {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_CONSTANT:
 					switch ($char) {
 						case ')':
 							if ($level == 0) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_CLOSE_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_CLOSE_BRACE,
 								$char, $this->pos, 1
 							);
 							$level--;
 							$state = self::STATE_AFTER_CLOSE_BRACE;
 							break;
+
 						default:
 							if ($this->parseUsing($this->binaryOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_BINARY_OPERATOR;
 								break;
 							}
@@ -382,34 +445,35 @@ class CTriggerExpression {
 							}
 
 							if ($this->parseUsing($this->logicalOperatorParser,
-									CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR)) {
-
+									CTriggerExprParserResult::TOKEN_TYPE_OPERATOR)) {
 								$state = self::STATE_AFTER_LOGICAL_OPERATOR;
 							}
 							else {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_NOT_OPERATOR:
 					switch ($char) {
 						case '-':
 							if (!$afterSpace) {
 								break 3;
 							}
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPERATOR,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPERATOR,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_MINUS_OPERATOR;
 							break;
+
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
 							$level++;
 							break;
+
 						default:
 							if (!$afterSpace) {
 								break 3;
@@ -422,17 +486,18 @@ class CTriggerExpression {
 								break 3;
 							}
 					}
-
 					break;
+
 				case self::STATE_AFTER_MINUS_OPERATOR:
 					switch ($char) {
 						case '(':
-							$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_OPEN_BRACE,
+							$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_OPEN_BRACE,
 								$char, $this->pos, 1
 							);
 							$state = self::STATE_AFTER_OPEN_BRACE;
 							$level++;
 							break;
+
 						default:
 							if ($this->parseConstant()) {
 								$state = self::STATE_AFTER_CONSTANT;
@@ -441,7 +506,6 @@ class CTriggerExpression {
 								break 3;
 							}
 					}
-
 					break;
 			}
 
@@ -450,15 +514,27 @@ class CTriggerExpression {
 		}
 
 		if ($this->pos == 0) {
-			$this->error = _('Incorrect trigger expression.');
+			$this->error = $this->options['calculated']
+				? _('incorrect calculated item formula')
+				: _('Incorrect trigger expression.');
 			$this->isValid = false;
 		}
 
-		if ($level != 0 || isset($this->expression[$this->pos])
-				|| ($state != self::STATE_AFTER_CLOSE_BRACE && $state != self::STATE_AFTER_CONSTANT)) {
+		$errors = array_filter([
+			($level != 0) ? self::ERROR_LEVEL : 0,
+			($state != self::STATE_AFTER_CLOSE_BRACE && $state != self::STATE_AFTER_CONSTANT)
+				? self::ERROR_UNEXPECTED_ENDING : 0,
+			isset($this->expression[$this->pos]) ? self::ERROR_UNPARSED_CONTENT : 0
+		]);
+		$error = reset($errors);
 
-			$this->error = _('Incorrect trigger expression.').' '._s('Check expression part starting from "%1$s".',
-					substr($this->expression, $this->pos == 0 ? 0 : $this->pos - 1));
+		if ($error) {
+			$exp_part = substr($this->expression, ($this->pos == 0) ? 0 : $this->pos - 1);
+			$this->error = $this->options['calculated']
+				? _s('incorrect calculated item formula starting from "%1$s"', $exp_part)
+				: _('Incorrect trigger expression.').' '._s('Check expression part starting from "%1$s".', $exp_part);
+			$this->error_type = $error;
+			$this->error_pos = $this->pos;
 			$this->isValid = false;
 
 			return false;
@@ -510,7 +586,7 @@ class CTriggerExpression {
 	 * constant.
 	 *
 	 * The constant can be:
-	 *  - trigger function like {host:item[].func()}
+	 *  - trigger function like {host:item[].func()}; can be without host:item pair
 	 *  - floating point number; can be with suffix [KMGTsmhdw]
 	 *  - macro like {TRIGGER.VALUE}
 	 *  - user macro like {$MACRO}
@@ -520,22 +596,91 @@ class CTriggerExpression {
 	 * @return bool  Returns true if parsed successfully, false otherwise.
 	 */
 	private function parseConstant() {
-		if ($this->parseFunctionMacro() || $this->parseNumber()
-				|| $this->parseUsing($this->user_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_USER_MACRO)
-				|| $this->parseUsing($this->macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_MACRO)) {
+		if ($this->parseNumber() || $this->parseString()
+				|| $this->parseUsing($this->user_macro_parser, CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO)) {
+			return true;
+		}
+
+		if ($this->options['calculated']) {
+			if ($this->parseFunction()) {
+				return true;
+			}
+		}
+		elseif ($this->parseFunctionMacro()
+				|| $this->parseUsing($this->macro_parser, CTriggerExprParserResult::TOKEN_TYPE_MACRO)) {
 			return true;
 		}
 
 		// LLD macro support for trigger prototypes.
 		if ($this->options['lldmacros']) {
-			if ($this->parseUsing($this->lld_macro_parser, CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO)
+			if ($this->parseUsing($this->lld_macro_parser, CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO)
 					|| $this->parseUsing($this->lld_macro_function_parser,
-							CTriggerExpressionParserResult::TOKEN_TYPE_LLD_MACRO)) {
+							CTriggerExprParserResult::TOKEN_TYPE_LLD_MACRO)) {
 				return true;
 			}
 		}
 
-		return false;
+		return ($this->options['allow_func_only'] && $this->parseFunctionOnly());
+	}
+
+	/**
+	 * Parses a trigger function in the trigger expression and moves a current position ($this->pos) on a last symbol of
+	 * the trigger function.
+	 *
+	 * @return bool  Returns true if parsed successfully, false otherwise.
+	 */
+	private function parseFunctionOnly() {
+		$pos = $this->pos;
+
+		if ($this->expression[$pos] !== '{') {
+			return false;
+		}
+
+		$pos++;
+
+		if ($this->function_parser->parse($this->expression, $pos) == CParser::PARSE_FAIL) {
+			return false;
+		}
+
+		$pos += $this->function_parser->getLength();
+
+		if (isset($this->expression[$pos]) && $this->expression[$pos] !== '}') {
+			return false;
+		}
+
+		$function_param_list = [];
+
+		for ($n = 0; $n < $this->function_parser->getParamsNum(); $n++) {
+			$function_param_list[] = $this->function_parser->getParam($n);
+		}
+
+		$expression = substr($this->expression, $this->pos, $pos + 1 - $this->pos);
+
+		$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO,
+			$expression, $this->pos, $this->function_parser->getLength() + 2,
+			[
+				'host' => '',
+				'item' => '',
+				'function' => $this->function_parser->getMatch(),
+				'functionName' => $this->function_parser->getFunction(),
+				'functionParams' => $function_param_list
+			]
+		);
+
+		$this->expressions[] = [
+			'expression' => $expression,
+			'pos' => $this->pos,
+			'host' => '',
+			'item' => '',
+			'function' => $this->function_parser->getMatch(),
+			'functionName' => $this->function_parser->getFunction(),
+			'functionParam' => $this->function_parser->getParameters(),
+			'functionParamList' => $function_param_list
+		];
+
+		$this->pos = $pos;
+
+		return true;
 	}
 
 	/**
@@ -545,6 +690,21 @@ class CTriggerExpression {
 	 * @return bool returns true if parsed successfully, false otherwise
 	 */
 	private function parseFunctionMacro() {
+		if ($this->options['collapsed_expression']) {
+			return $this->parseUsing($this->functionid_parser, CTriggerExprParserResult::TOKEN_TYPE_FUNCTIONID_MACRO);
+		}
+		else {
+			return $this->parseSimpleMacro();
+		}
+	}
+
+	/**
+	 * Parses a simple macro constant {host:key.func()} in the trigger expression and
+	 * moves a current position ($this->pos) on a last symbol of the macro
+	 *
+	 * @return bool returns true if parsed successfully, false otherwise
+	 */
+	private function parseSimpleMacro() {
 		$startPos = $this->pos;
 
 		if ($this->function_macro_parser->parse($this->expression, $this->pos) == CParser::PARSE_FAIL) {
@@ -563,7 +723,7 @@ class CTriggerExpression {
 			$function_param_list[] = $this->function_parser->getParam($n);
 		}
 
-		$this->result->addToken(CTriggerExpressionParserResult::TOKEN_TYPE_FUNCTION_MACRO,
+		$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_FUNCTION_MACRO,
 			$this->function_macro_parser->getMatch(), $startPos, $this->function_macro_parser->getLength(),
 			[
 				'host' => $this->function_macro_parser->getHost(),
@@ -588,59 +748,144 @@ class CTriggerExpression {
 		return true;
 	}
 
+	/**
+	 * Parses a function in the calculated item formula and moves
+	 * a current position ($this->pos) on a last symbol of the macro.
+	 *
+	 * @return bool  Returns true if parsed successfully, false otherwise.
+	 */
+	private function parseFunction() {
+		$startPos = $this->pos;
+
+		if ($this->function_parser->parse($this->expression, $this->pos) == CParser::PARSE_FAIL) {
+			return false;
+		}
+
+		$this->pos += $this->function_parser->getLength() - 1;
+
+		$function_param_list = [];
+
+		for ($n = 0; $n < $this->function_parser->getParamsNum(); $n++) {
+			$function_param_list[] = $this->function_parser->getParam($n);
+		}
+
+		$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_FUNCTION,
+			$this->function_parser->getMatch(), $startPos, $this->function_parser->getLength(),
+			[
+				'functionName' => $this->function_parser->getFunction(),
+				'functionParams' => $function_param_list
+			]
+		);
+
+		$this->expressions[] = [
+			'expression' => $this->function_parser->getMatch(),
+			'pos' => $startPos,
+			'functionName' => $this->function_parser->getFunction(),
+			'functionParam' => $this->function_parser->getParameters(),
+			'functionParamList' => $function_param_list
+		];
+
+		return true;
+	}
+
 	/**
 	 * Parses a number constant in the trigger expression and
 	 * moves a current position ($this->pos) on a last symbol of the number
 	 *
-	 * comments: !!! Don't forget sync code with C !!!
-	 *
 	 * @return bool returns true if parsed successfully, false otherwise
 	 */
 	private function parseNumber() {
-		$j = $this->pos;
-		$digits = 0;
-		$dots = 0;
-
-		while (isset($this->expression[$j])) {
-			if ($this->expression[$j] >= '0' && $this->expression[$j] <= '9') {
-				$digits++;
-				$j++;
-				continue;
-			}
-
-			if ($this->expression[$j] === '.') {
-				$dots++;
-				$j++;
-				continue;
-			}
-
-			break;
-		}
-
-		if ($digits == 0 || $dots > 1) {
+		if ($this->number_parser->parse($this->expression, $this->pos) == CParser::PARSE_FAIL) {
 			return false;
 		}
 
-		// check for an optional suffix
-		$suffix = null;
-		if (isset($this->expression[$j])
-				&& strpos(ZBX_BYTE_SUFFIXES.ZBX_TIME_SUFFIXES, $this->expression[$j]) !== false) {
-			$suffix = $this->expression[$j];
-			$j++;
+		$value = $this->number_parser->calcValue();
+		if (abs($value) == INF) {
+			return false;
 		}
 
-		$numberLength = $j - $this->pos;
+		$token_data = [
+			'suffix' => $this->number_parser->getSuffix()
+		];
 
 		$this->result->addToken(
-			CTriggerExpressionParserResult::TOKEN_TYPE_NUMBER,
-			substr($this->expression, $this->pos, $numberLength),
+			CTriggerExprParserResult::TOKEN_TYPE_NUMBER,
+			$this->number_parser->getMatch(),
 			$this->pos,
-			$numberLength,
-			['suffix' => $suffix]
+			$this->number_parser->getLength(),
+			$token_data
 		);
 
-		$this->pos = $j - 1;
+		$this->pos += $this->number_parser->getLength() - 1;
 
 		return true;
 	}
+
+	/**
+	 * Parses a quoted string constant in the trigger expression and moves a current position ($this->pos) on a last
+	 * symbol of the string.
+	 *
+	 * @return bool returns true if parsed successfully, false otherwise
+	 */
+	private function parseString() {
+		if (!preg_match('/^"([^"\\\\]|\\\\["\\\\])*"/', substr($this->expression, $this->pos), $matches)) {
+			return false;
+		}
+
+		$len = strlen($matches[0]);
+
+		$this->result->addToken(CTriggerExprParserResult::TOKEN_TYPE_STRING, $matches[0], $this->pos, $len,
+			['string' => self::unquoteString($matches[0])]
+		);
+
+		$this->pos += $len - 1;
+
+		return true;
+	}
+
+	/**
+	 * Unquoting quoted string $value.
+	 *
+	 * @param string $value
+	 *
+	 * @return string
+	 */
+	public static function unquoteString(string $value): string {
+		return strtr(substr($value, 1, -1), ['\\"' => '"', '\\\\' => '\\']);
+	}
+
+	/**
+	 * Quoting $value if it contains a non numeric value.
+	 *
+	 * @param string $value
+	 * @param bool   $allow_macros
+	 * @param bool   $force
+	 *
+	 * @return string
+	 */
+	public static function quoteString(string $value, bool $allow_macros = true, bool $force = false): string {
+		if (!$force) {
+			$number_parser = new CNumberParser(['with_suffix' => true]);
+
+			if ($number_parser->parse($value) == CParser::PARSE_SUCCESS) {
+				return $value;
+			}
+
+			if ($allow_macros) {
+				$user_macro_parser = new CUserMacroParser();
+				$macro_parser = new CMacroParser(['macros' => ['{TRIGGER.VALUE}']]);
+				$lld_macro_parser = new CLLDMacroParser();
+				$lld_macro_function_parser = new CLLDMacroFunctionParser;
+
+				if ($user_macro_parser->parse($value) == CParser::PARSE_SUCCESS
+						|| $macro_parser->parse($value) == CParser::PARSE_SUCCESS
+						|| $lld_macro_parser->parse($value) == CParser::PARSE_SUCCESS
+						|| $lld_macro_function_parser->parse($value) == CParser::PARSE_SUCCESS) {
+					return $value;
+				}
+			}
+		}
+
+		return '"'.strtr($value, ['\\' => '\\\\', '"' => '\\"']).'"';
+	}
 }
