@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -33,19 +33,15 @@ function eventSource($source = null) {
 	$sources = [
 		EVENT_SOURCE_TRIGGERS => _('trigger'),
 		EVENT_SOURCE_DISCOVERY => _('discovery'),
-		EVENT_SOURCE_AUTO_REGISTRATION => _('auto registration'),
+		EVENT_SOURCE_AUTOREGISTRATION => _('autoregistration'),
 		EVENT_SOURCE_INTERNAL => _x('internal', 'event source')
 	];
 
 	if ($source === null) {
 		return $sources;
 	}
-	elseif (isset($sources[$source])) {
-		return $sources[$source];
-	}
-	else {
-		return _('Unknown');
-	}
+
+	return array_key_exists($source, $sources) ?  $sources[$source] : _('Unknown');
 }
 
 /**
@@ -63,7 +59,7 @@ function eventObject($object = null) {
 		EVENT_OBJECT_TRIGGER => _('trigger'),
 		EVENT_OBJECT_DHOST => _('discovered host'),
 		EVENT_OBJECT_DSERVICE => _('discovered service'),
-		EVENT_OBJECT_AUTOREGHOST => _('auto-registered host'),
+		EVENT_OBJECT_AUTOREGHOST => _('autoregistered host'),
 		EVENT_OBJECT_ITEM => _('item'),
 		EVENT_OBJECT_LLDRULE => _('low-level discovery rule')
 	];
@@ -89,7 +85,7 @@ function eventSourceObjects() {
 		['source' => EVENT_SOURCE_TRIGGERS, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DHOST],
 		['source' => EVENT_SOURCE_DISCOVERY, 'object' => EVENT_OBJECT_DSERVICE],
-		['source' => EVENT_SOURCE_AUTO_REGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
+		['source' => EVENT_SOURCE_AUTOREGISTRATION, 'object' => EVENT_OBJECT_AUTOREGHOST],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_TRIGGER],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_ITEM],
 		['source' => EVENT_SOURCE_INTERNAL, 'object' => EVENT_OBJECT_LLDRULE]
@@ -104,12 +100,11 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 		return 0;
 	}
 
-	$config = select_config();
 	$options = [
 		'output' => ['triggerid'],
 		'monitored' => 1,
 		'skipDependent' => 1,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 	if (!is_null($value_trigger)) {
 		$options['filter'] = ['value' => $value_trigger];
@@ -139,33 +134,56 @@ function get_events_unacknowledged($db_element, $value_trigger = null, $value_ev
 
 /**
  *
- * @param array  $event								An array of event data.
- * @param string $event['eventid']					Event ID.
- * @param string $event['correlationid']			OK Event correlation ID.
- * @param string $event['userid]					User ID who generated the OK event.
- * @param string $event['name']						Event name.
- * @param string $event['acknowledged']				State of acknowledgement.
- * @param string $backurl							A link back after acknowledgement has been clicked.
+ * @param array  $event                              An array of event data.
+ * @param string $event['eventid']                   Event ID.
+ * @param string $event['r_eventid']                 OK event ID.
+ * @param string $event['objectid']                  Object ID.
+ * @param string $event['correlationid']             OK Event correlation ID.
+ * @param string $event['userid']                    User ID who generated the OK event.
+ * @param string $event['name']                      Event name.
+ * @param string $event['acknowledged']              State of acknowledgement.
+ * @param array  $event['acknowledges']              List of problem updates.
+ * @param string $event['acknowledges'][]['action']  Action performed in update.
+ * @param CCOl   $event['opdata']                    Operational data with expanded macros.
+ * @param string $event['comments']                  Trigger description with expanded macros.
+ * @param array  $allowed                            An array of user role rules.
+ * @param bool   $allowed['ui_correlation']          Whether user is allowed to visit event correlation page.
+ * @param bool   $allowed['add_comments']            Whether user is allowed to add problems comments.
+ * @param bool   $allowed['change_severity']         Whether user is allowed to change problems severity.
+ * @param bool   $allowed['acknowledge']             Whether user is allowed to acknowledge problems.
+ * @param bool   $allowed['close']                   Whether user is allowed to close problems.
  *
  * @return CTableInfo
  */
-function make_event_details($event, $backurl) {
-	$event_update_url = (new CUrl('zabbix.php'))
-		->setArgument('action', 'acknowledge.edit')
-		->setArgument('eventids', [$event['eventid']])
-		->setArgument('backurl', $backurl)
-		->getUrl();
+function make_event_details(array $event, array $allowed) {
+	$can_be_closed = $allowed['close'];
 
-	$config = select_config();
+	if ($event['r_eventid'] != 0) {
+		$can_be_closed = false;
+	}
+	else {
+		foreach ($event['acknowledges'] as $acknowledge) {
+			if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
+				$can_be_closed = false;
+				break;
+			}
+		}
+	}
+
+	$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 
 	$table = (new CTableInfo())
 		->addRow([
 			_('Event'),
 			$event['name']
 		])
+		->addRow([
+			_('Operational data'),
+			$event['opdata']
+		])
 		->addRow([
 			_('Severity'),
-			getSeverityCell($event['severity'], $config)
+			getSeverityCell($event['severity'])
 		])
 		->addRow([
 			_('Time'),
@@ -173,9 +191,14 @@ function make_event_details($event, $backurl) {
 		])
 		->addRow([
 			_('Acknowledged'),
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $event_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT)
+			($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge'] || $can_be_closed)
+				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+				)
 		]);
 
 	if ($event['r_eventid'] != 0) {
@@ -186,7 +209,7 @@ function make_event_details($event, $backurl) {
 			]);
 
 			if ($correlations) {
-				if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
+				if ($allowed['ui_correlation']) {
 					$correlation_name = (new CLink($correlations[0]['name'],
 						(new CUrl('correlation.php'))
 							->setArgument('correlationid', $correlations[0]['correlationid'])
@@ -221,7 +244,7 @@ function make_event_details($event, $backurl) {
 					$table->addRow([_('Resolved by'), getUserFullname($user[0])]);
 				}
 				else {
-					$table->addRow([_('Resolved by'), _('User')]);
+					$table->addRow([_('Resolved by'), _('Inaccessible user')]);
 				}
 			}
 		}
@@ -232,14 +255,27 @@ function make_event_details($event, $backurl) {
 
 	$tags = makeTags([$event]);
 
-	$table->addRow([_('Tags'), $tags[$event['eventid']]]);
+	$table
+		->addRow([_('Tags'), $tags[$event['eventid']]])
+		->addRow([_('Description'), (new CDiv(zbx_str2links($event['comments'])))]);
 
 	return $table;
 }
 
-function make_small_eventlist($startEvent, $backurl) {
-	$config = select_config();
-
+/**
+ *
+ * @param array  $startEvent                  An array of event data.
+ * @param string $startEvent['eventid']       Event ID.
+ * @param string $startEvent['objectid']      Object ID.
+ * @param array  $allowed                     An array of user role rules.
+ * @param bool   $allowed['add_comments']     Whether user is allowed to add problems comments.
+ * @param bool   $allowed['change_severity']  Whether user is allowed to change problems severity.
+ * @param bool   $allowed['acknowledge']      Whether user is allowed to acknowledge problems.
+ * @param bool   $allowed['close']            Whether user is allowed to close problems.
+ *
+ * @return CTableInfo
+ */
+function make_small_eventlist(array $startEvent, array $allowed) {
 	$table = (new CTableInfo())
 		->setHeader([
 			_('Time'),
@@ -301,14 +337,6 @@ function make_small_eventlist($startEvent, $backurl) {
 		])
 		: [];
 
-	$severity_config = [
-		'severity_name_0' => $config['severity_name_0'],
-		'severity_name_1' => $config['severity_name_1'],
-		'severity_name_2' => $config['severity_name_2'],
-		'severity_name_3' => $config['severity_name_3'],
-		'severity_name_4' => $config['severity_name_4'],
-		'severity_name_5' => $config['severity_name_5']
-	];
 	$actions = getEventsActionsIconsData($events, $triggers, $r_events);
 	$users = API::User()->get([
 		'output' => ['alias', 'name', 'surname'],
@@ -316,7 +344,7 @@ function make_small_eventlist($startEvent, $backurl) {
 		'preservekeys' => true
 	]);
 	$mediatypes = API::Mediatype()->get([
-		'output' => ['description', 'maxattempts'],
+		'output' => ['name', 'maxattempts'],
 		'mediatypeids' => array_keys($actions['mediatypeids']),
 		'preservekeys' => true
 	]);
@@ -326,12 +354,21 @@ function make_small_eventlist($startEvent, $backurl) {
 			? zbx_date2age($event['clock'], $event['r_clock'])
 			: zbx_date2age($event['clock']);
 
-		if ($event['r_eventid'] == 0) {
+		$can_be_closed = $allowed['close'];
+
+		if ($event['r_eventid'] != 0) {
+			$value = TRIGGER_VALUE_FALSE;
+			$value_str = _('RESOLVED');
+			$value_clock = $event['r_clock'];
+			$can_be_closed = false;
+		}
+		else {
 			$in_closing = false;
 
 			foreach ($event['acknowledges'] as $acknowledge) {
 				if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
 					$in_closing = true;
+					$can_be_closed = false;
 					break;
 				}
 			}
@@ -340,26 +377,26 @@ function make_small_eventlist($startEvent, $backurl) {
 			$value_str = $in_closing ? _('CLOSING') : _('PROBLEM');
 			$value_clock = $in_closing ? time() : $event['clock'];
 		}
-		else {
-			$value = TRIGGER_VALUE_FALSE;
-			$value_str = _('RESOLVED');
-			$value_clock = $event['r_clock'];
-		}
 
+		$is_acknowledged = ($event['acknowledged'] == EVENT_ACKNOWLEDGED);
 		$cell_status = new CSpan($value_str);
 
 		/*
 		 * Add colors to span depending on configuration and trigger parameters. No blinking added to status,
 		 * since the page is not on autorefresh.
 		 */
-		addTriggerValueStyle($cell_status, $value, $value_clock, $event['acknowledged'] == EVENT_ACKNOWLEDGED);
+		addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
-		// Create link to Problem update page.
-		$problem_update_url = (new CUrl('zabbix.php'))
-			->setArgument('action', 'acknowledge.edit')
-			->setArgument('eventids', [$event['eventid']])
-			->setArgument('backurl', $backurl)
-			->getUrl();
+		// Create acknowledge link.
+		$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+				|| $can_be_closed)
+			? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+				->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+				->addClass(ZBX_STYLE_LINK_ALT)
+				->onClick('acknowledgePopUp('.json_encode(['eventids' => [$event['eventid']]]).', this);')
+			: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+				$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+			);
 
 		$table->addRow([
 			(new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $event['clock']),
@@ -373,10 +410,8 @@ function make_small_eventlist($startEvent, $backurl) {
 			$cell_status,
 			zbx_date2age($event['clock']),
 			$duration,
-			(new CLink($event['acknowledged'] == EVENT_ACKNOWLEDGED ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($event['acknowledged'] == EVENT_ACKNOWLEDGED ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT),
-			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users, $severity_config)
+			$problem_update_link,
+			makeEventActionsIcons($event['eventid'], $actions['data'], $mediatypes, $users)
 		]);
 	}
 
@@ -386,33 +421,37 @@ function make_small_eventlist($startEvent, $backurl) {
 /**
  * Create table with trigger description and events.
  *
- * @param array  $trigger                    An array of trigger data.
- * @param string $trigger['triggerid']       Trigger ID to select events.
- * @param string $trigger['description']     Trigger description.
- * @param string $trigger['url']             Trigger URL.
+ * @param array  $trigger                     An array of trigger data.
+ * @param string $trigger['triggerid']        Trigger ID to select events.
+ * @param string $trigger['comments']         Trigger description.
+ * @param string $trigger['url']              Trigger URL.
  * @param string $eventid_till
- * @param string $backurl                    URL to return to.
- * @param bool   $show_timeline              Show time line flag.
- * @param int    $show_tags                  Show tags flag. Possible values:
- *                                             - PROBLEMS_SHOW_TAGS_NONE;
- *                                             - PROBLEMS_SHOW_TAGS_1;
- *                                             - PROBLEMS_SHOW_TAGS_2;
- *                                             - PROBLEMS_SHOW_TAGS_3 (default).
- * @param array  $filter_tags                An array of tag filtering data.
- * @param string $filter_tags[]['tag']       Tag name.
- * @param int    $filter_tags[]['operator']  Tag operator.
- * @param string $filter_tags[]['value']     Tag value.
- * @param int    $tag_name_format            Tag name format. Possible values:
- *                                             - PROBLEMS_TAG_NAME_FULL (default);
- *                                             - PROBLEMS_TAG_NAME_SHORTENED;
- *                                             - PROBLEMS_TAG_NAME_NONE.
- * @param string $tag_priority               A list of comma-separated tag names.
+ * @param array  $allowed                     An array of user role rules.
+ * @param bool   $allowed['ui_problems']      Whether user is allowed to visit problems page.
+ * @param bool   $allowed['add_comments']     Whether user is allowed to add problems comments.
+ * @param bool   $allowed['change_severity']  Whether user is allowed to change problems severity.
+ * @param bool   $allowed['acknowledge']      Whether user is allowed to acknowledge problems.
+ * @param bool   $allowed['close']            Whether user is allowed to close problems.
+ * @param bool   $show_timeline               Show time line flag.
+ * @param int    $show_tags                   Show tags flag. Possible values:
+ *                                              - PROBLEMS_SHOW_TAGS_NONE;
+ *                                              - PROBLEMS_SHOW_TAGS_1;
+ *                                              - PROBLEMS_SHOW_TAGS_2;
+ *                                              - PROBLEMS_SHOW_TAGS_3 (default).
+ * @param array  $filter_tags                 An array of tag filtering data.
+ * @param string $filter_tags[]['tag']        Tag name.
+ * @param int    $filter_tags[]['operator']   Tag operator.
+ * @param string $filter_tags[]['value']      Tag value.
+ * @param int    $tag_name_format             Tag name format. Possible values:
+ *                                              - PROBLEMS_TAG_NAME_FULL (default);
+ *                                              - PROBLEMS_TAG_NAME_SHORTENED;
+ *                                              - PROBLEMS_TAG_NAME_NONE.
+ * @param string $tag_priority                A list of comma-separated tag names.
  *
  * @return CDiv
  */
-function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline = true,
-		$show_tags = PROBLEMS_SHOW_TAGS_3, array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL,
-		$tag_priority = '') {
+function make_popup_eventlist($trigger, $eventid_till, $allowed, $show_timeline = true, $show_tags = PROBLEMS_SHOW_TAGS_3,
+		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
 	// Show trigger description and URL.
 	$div = new CDiv();
 
@@ -426,7 +465,7 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 	}
 
 	if ($trigger['url'] !== '') {
-		$trigger_url = CHtmlUrlValidator::validate($trigger['url'])
+		$trigger_url = CHtmlUrlValidator::validate($trigger['url'], ['allow_user_macro' => false])
 			? $trigger['url']
 			: 'javascript: alert(\''._s('Provided URL "%1$s" is invalid.', zbx_jsvalue($trigger['url'], false, false)).
 				'\');';
@@ -507,9 +546,11 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 			$tags = makeTags($problems, true, 'eventid', $show_tags, $filter_tags, $tag_name_format, $tag_priority);
 		}
 
-		$url_details = (new CUrl('tr_events.php'))
-			->setArgument('triggerid', '')
-			->setArgument('eventid', '');
+		$url_details = $allowed['ui_problems']
+			? (new CUrl('tr_events.php'))
+				->setArgument('triggerid', '')
+				->setArgument('eventid', '')
+			: null;
 
 		foreach ($problems as $problem) {
 			if (array_key_exists($problem['r_eventid'], $r_events)) {
@@ -523,10 +564,13 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$problem['userid'] = 0;
 			}
 
+			$can_be_closed = $allowed['close'];
+
 			if ($problem['r_eventid'] != 0) {
 				$value = TRIGGER_VALUE_FALSE;
 				$value_str = _('RESOLVED');
 				$value_clock = $problem['r_clock'];
+				$can_be_closed = false;
 			}
 			else {
 				$in_closing = false;
@@ -534,6 +578,7 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				foreach ($problem['acknowledges'] as $acknowledge) {
 					if (($acknowledge['action'] & ZBX_PROBLEM_UPDATE_CLOSE) == ZBX_PROBLEM_UPDATE_CLOSE) {
 						$in_closing = true;
+						$can_be_closed = false;
 						break;
 					}
 				}
@@ -543,19 +588,28 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$value_clock = $in_closing ? time() : $problem['clock'];
 			}
 
-			$url_details
-				->setArgument('triggerid', $problem['objectid'])
-				->setArgument('eventid', $problem['eventid']);
-
 			$cell_clock = ($problem['clock'] >= $today)
 				? zbx_date2str(TIME_FORMAT_SECONDS, $problem['clock'])
 				: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['clock']);
-			$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+
+			if ($url_details !== null) {
+				$url_details
+					->setArgument('triggerid', $problem['objectid'])
+					->setArgument('eventid', $problem['eventid']);
+				$cell_clock = new CCol(new CLink($cell_clock, $url_details));
+			}
+			else {
+				$cell_clock = new CCol($cell_clock);
+			}
+
 			if ($problem['r_eventid'] != 0) {
 				$cell_r_clock = ($problem['r_clock'] >= $today)
 					? zbx_date2str(TIME_FORMAT_SECONDS, $problem['r_clock'])
 					: zbx_date2str(DATE_TIME_FORMAT_SECONDS, $problem['r_clock']);
-				$cell_r_clock = (new CCol(new CLink($cell_r_clock, $url_details)))
+				$cell_r_clock = (new CCol(($url_details !== null)
+					? new CLink($cell_r_clock, $url_details)
+					: $cell_r_clock
+				))
 					->addClass(ZBX_STYLE_NOWRAP)
 					->addClass(ZBX_STYLE_RIGHT);
 			}
@@ -563,10 +617,11 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 				$cell_r_clock = '';
 			}
 
+			$is_acknowledged = ($problem['acknowledged'] == EVENT_ACKNOWLEDGED);
 			$cell_status = new CSpan($value_str);
 
 			// add colors and blinking to span depending on configuration and trigger parameters
-			addTriggerValueStyle($cell_status, $value, $value_clock, $problem['acknowledged'] == EVENT_ACKNOWLEDGED);
+			addTriggerValueStyle($cell_status, $value, $value_clock, $is_acknowledged);
 
 			if ($show_timeline) {
 				if ($last_clock != 0) {
@@ -591,16 +646,15 @@ function make_popup_eventlist($trigger, $eventid_till, $backurl, $show_timeline
 			}
 
 			// Create acknowledge link.
-			$problem_update_url = (new CUrl('zabbix.php'))
-				->setArgument('action', 'acknowledge.edit')
-				->setArgument('eventids', [$problem['eventid']])
-				->setArgument('backurl', $backurl)
-				->getUrl();
-
-			$acknowledged = $problem['acknowledged'] == EVENT_ACKNOWLEDGED;
-			$problem_update_link = (new CLink($acknowledged ? _('Yes') : _('No'), $problem_update_url))
-				->addClass($acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
-				->addClass(ZBX_STYLE_LINK_ALT);
+			$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
+					|| $can_be_closed)
+				? (new CLink($is_acknowledged ? _('Yes') : _('No')))
+					->addClass($is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED)
+					->addClass(ZBX_STYLE_LINK_ALT)
+					->onClick('acknowledgePopUp('.json_encode(['eventids' => [$problem['eventid']]]).', this);')
+				: (new CSpan($is_acknowledged ? _('Yes') : _('No')))->addClass(
+					$is_acknowledged ? ZBX_STYLE_GREEN : ZBX_STYLE_RED
+				);
 
 			$table->addRow(array_merge($row, [
 				$cell_r_clock,
@@ -683,33 +737,36 @@ function orderEventTagsByPriority(array $event_tags, array $priorities) {
  * Create element with tags.
  *
  * @param array  $list
- * @param string $list[]['eventid'|'triggerid']
+ * @param string $list[][$key]
  * @param array  $list[]['tags']
  * @param string $list[]['tags'][]['tag']
  * @param string $list[]['tags'][]['value']
  * @param bool   $html
- * @param string $key                            Name of tags source ID. Possible values:
- *                                                - 'eventid' - for events and problems (default);
- *                                                - 'triggerid' - for triggers.
- * @param int    $list_tag_count                 Maximum number of tags to display.
- * @param array  $filter_tags                    An array of tag filtering data.
+ * @param string $key                        Name of tag source ID. Possible values:
+ *                                            - 'eventid' - for events and problems (default);
+ *                                            - 'hostid' - for hosts and host prototypes;
+ *                                            - 'templateid' - for templates;
+ *                                            - 'triggerid' - for triggers.
+ * @param int    $list_tag_count             Maximum number of tags to display.
+ * @param array  $filter_tags                An array of tag filtering data.
  * @param string $filter_tags[]['tag']
  * @param int    $filter_tags[]['operator']
  * @param string $filter_tags[]['value']
- * @param int    $tag_name_format                Tag name format. Possible values:
- *                                                - PROBLEMS_TAG_NAME_FULL (default);
- *                                                - PROBLEMS_TAG_NAME_SHORTENED;
- *                                                - PROBLEMS_TAG_NAME_NONE.
- * @param string $tag_priority                   A list of comma-separated tag names.
+ * @param int    $tag_name_format            Tag name format. Possible values:
+ *                                            - PROBLEMS_TAG_NAME_FULL (default);
+ *                                            - PROBLEMS_TAG_NAME_SHORTENED;
+ *                                            - PROBLEMS_TAG_NAME_NONE.
+ * @param string $tag_priority               A list of comma-separated tag names.
  *
  * @return array
  */
-function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
-		array $filter_tags = [], $tag_name_format = PROBLEMS_TAG_NAME_FULL, $tag_priority = '') {
+function makeTags(array $list, bool $html = true, string $key = 'eventid', int $list_tag_count = ZBX_TAG_COUNT_DEFAULT,
+		array $filter_tags = [], int $tag_name_format = PROBLEMS_TAG_NAME_FULL, string $tag_priority = ''): array {
 	$tags = [];
 
 	if ($html) {
 		// Convert $filter_tags to a more usable format.
+
 		$f_tags = [];
 
 		foreach ($filter_tags as $tag) {
@@ -726,10 +783,14 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 	}
 
 	foreach ($list as $element) {
-		CArrayHelper::sort($element['tags'], ['tag', 'value']);
-
 		$tags[$element[$key]] = [];
 
+		if (!$element['tags']) {
+			continue;
+		}
+
+		CArrayHelper::sort($element['tags'], ['tag', 'value']);
+
 		if ($html) {
 			// Show first n tags and "..." with hint box if there are more.
 
@@ -748,7 +809,9 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 					$tags[$element[$key]][] = (new CSpan($value))
 						->addClass(ZBX_STYLE_TAG)
 						->setHint(getTagString($tag));
+
 					$tags_shown++;
+
 					if ($tags_shown >= $list_tag_count) {
 						break;
 					}
@@ -771,7 +834,7 @@ function makeTags(array $list, $html = true, $key = 'eventid', $list_tag_count =
 					(new CButton(null))
 						->addClass(ZBX_STYLE_ICON_WZRD_ACTION)
 						->setHint(new CDiv($hint_content), '', true, 'max-width: 500px')
-					))->addClass(ZBX_STYLE_REL_CONTAINER);
+				))->addClass(ZBX_STYLE_REL_CONTAINER);
 			}
 		}
 		else {
@@ -808,74 +871,3 @@ function getTagString(array $tag, $tag_name_format = PROBLEMS_TAG_NAME_FULL) {
 			return $tag['tag'].(($tag['value'] === '') ? '' : ': '.$tag['value']);
 	}
 }
-
-function getLastEvents($options) {
-	if (!isset($options['limit'])) {
-		$options['limit'] = 15;
-	}
-
-	$triggerOptions = [
-		'output' => ['triggerid', 'priority'],
-		'filter' => [],
-		'skipDependent' => 1,
-		'selectHosts' => ['hostid', 'name'],
-		'sortfield' => 'lastchange',
-		'sortorder' => ZBX_SORT_DOWN,
-		'limit' => $options['triggerLimit']
-	];
-
-	$eventOptions = [
-		'source' => EVENT_SOURCE_TRIGGERS,
-		'object' => EVENT_OBJECT_TRIGGER,
-		'output' => API_OUTPUT_EXTEND,
-		'sortfield' => ['clock', 'eventid'],
-		'sortorder' => ZBX_SORT_DOWN
-	];
-
-	if (isset($options['eventLimit'])) {
-		$eventOptions['limit'] = $options['eventLimit'];
-	}
-
-	if (isset($options['priority'])) {
-		$triggerOptions['filter']['priority'] = $options['priority'];
-	}
-	if (isset($options['monitored'])) {
-		$triggerOptions['monitored'] = $options['monitored'];
-	}
-	if (isset($options['lastChangeSince'])) {
-		$triggerOptions['lastChangeSince'] = $options['lastChangeSince'];
-		$eventOptions['time_from'] = $options['lastChangeSince'];
-	}
-	if (isset($options['value'])) {
-		$triggerOptions['filter']['value'] = $options['value'];
-		$eventOptions['value'] = $options['value'];
-	}
-	if (array_key_exists('suppressed', $options)) {
-		$eventOptions['suppressed'] = $options['suppressed'];
-	}
-
-	// triggers
-	$triggers = API::Trigger()->get($triggerOptions);
-	$triggers = zbx_toHash($triggers, 'triggerid');
-
-	// events
-	$eventOptions['objectids'] = zbx_objectValues($triggers, 'triggerid');
-	$events = API::Event()->get($eventOptions);
-
-	$sortClock = [];
-	$sortEvent = [];
-	foreach ($events as $enum => $event) {
-		if (!isset($triggers[$event['objectid']])) {
-			continue;
-		}
-
-		$events[$enum]['trigger'] = $triggers[$event['objectid']];
-		$events[$enum]['host'] = reset($events[$enum]['trigger']['hosts']);
-		$sortClock[$enum] = $event['clock'];
-		$sortEvent[$enum] = $event['eventid'];
-		$events[$enum]['trigger']['description'] = $event['name'];
-	}
-	array_multisort($sortClock, SORT_DESC, $sortEvent, SORT_DESC, $events);
-
-	return $events;
-}
