@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,31 +31,24 @@ if (!defined('PAGE_HEADER_LOADED')) {
 }
 
 // last page
-if (!defined('ZBX_PAGE_NO_MENU') && $page['file'] != 'profile.php') {
+if (!defined('ZBX_PAGE_NO_MENU')) {
 	CProfile::update('web.paging.lastpage', $page['file'], PROFILE_TYPE_STR);
 }
 
-if (CProfile::isModified()) {
-	DBstart();
-	$result = CProfile::flush();
-	DBend($result);
-}
-
 // end transactions if they have not been closed already
 if (isset($DB) && isset($DB['TRANSACTIONS']) && $DB['TRANSACTIONS'] != 0) {
 	error(_('Transaction has not been closed. Aborting...'));
 	DBend(false);
 }
 
-show_messages();
+// Display unexpected messages (if any) generated while processing the output.
+echo get_prepared_messages(['with_current_messages' => true]);
 
 if ($page['type'] == PAGE_TYPE_HTML) {
-	// end of article div
-	echo '</main>'."\n";
-	if (!defined('ZBX_PAGE_NO_MENU')) {
-		makePageFooter()->show();
-	}
+	makeServerStatusOutput()->show();
+
 	insertPagePostJs();
+
 	require_once 'include/views/js/common.init.js.php';
 
 	if (in_array($page['type'], [PAGE_TYPE_HTML_BLOCK, PAGE_TYPE_HTML])) {
@@ -63,11 +56,19 @@ if ($page['type'] == PAGE_TYPE_HTML) {
 				&& CWebUser::$data['debug_mode'] == GROUP_DEBUG_MODE_ENABLED) {
 			CProfiler::getInstance()->stop();
 			CProfiler::getInstance()->show();
-			makeDebugButton()->show();
+
+			(new CButton(null))
+				->addClass(ZBX_STYLE_BTN_DEBUG)
+				->show();
 		}
 	}
 
-	echo '</body></html>';
+	if (!defined('ZBX_PAGE_NO_MENU')) {
+		makePageFooter()->show();
+	}
+
+	echo '</div></body></html>';
 }
 
-exit;
+session_write_close();
+exit();
