@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,17 +19,18 @@
 **/
 
 
-$widget = (new CWidget())->setTitle(_('Screens').': '.$data['screen']['name']);
+/**
+ * @var CView $this
+ */
 
-if ($data['screen']['templateid']) {
-	$widget->addItem(get_header_host_table('screens', $data['screen']['templateid']));
-}
+$this->includeJsFile('monitoring.screen.js.php');
+
+$widget = (new CWidget())->setTitle(_('Screens').': '.$data['screen']['name']);
 
 $screenBuilder = new CScreenBuilder([
 	'isFlickerfree' => false,
 	'screen' => $data['screen'],
-	'mode' => SCREEN_MODE_EDIT,
-	'updateProfile' => false
+	'mode' => SCREEN_MODE_EDIT
 ]);
 
 $widget->addItem(
@@ -39,4 +40,4 @@ $widget->addItem(
 $screenBuilder->insertInitScreenJs($data['screenid']);
 $screenBuilder->insertProcessObjectsJs();
 
-return $widget;
+$widget->show();
