@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 $page['title'] = _('Configuration of network maps');
 $page['file'] = 'sysmaps.php';
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
-$page['scripts'] = ['multiselect.js'];
+$page['scripts'] = ['multiselect.js', 'class.tab-indicators.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -110,10 +110,16 @@ else {
 	$sysmap = [];
 }
 
+$allowed_edit = CWebUser::checkAccess(CRoleHelper::ACTIONS_EDIT_MAPS);
+
 /*
  * Actions
  */
 if (hasRequest('add') || hasRequest('update')) {
+	if (!$allowed_edit) {
+		access_deny(ACCESS_DENY_PAGE);
+	}
+
 	$map = [
 		'name' => getRequest('name'),
 		'width' => getRequest('width'),
@@ -224,7 +230,12 @@ if (hasRequest('add') || hasRequest('update')) {
 	}
 	show_messages($result, $messageSuccess, $messageFailed);
 }
-elseif ((hasRequest('delete') && hasRequest('sysmapid')) || (hasRequest('action') && getRequest('action') == 'map.massdelete')) {
+elseif ((hasRequest('delete') && hasRequest('sysmapid'))
+		|| (hasRequest('action') && getRequest('action') == 'map.massdelete')) {
+	if (!$allowed_edit) {
+		access_deny(ACCESS_DENY_PAGE);
+	}
+
 	$sysmapIds = getRequest('maps', []);
 
 	if (hasRequest('sysmapid')) {
@@ -263,6 +274,10 @@ elseif ((hasRequest('delete') && hasRequest('sysmapid')) || (hasRequest('action'
  * Display
  */
 if (hasRequest('form')) {
+	if (!$allowed_edit) {
+		access_deny(ACCESS_DENY_PAGE);
+	}
+
 	$current_userid = CWebUser::$data['userid'];
 	$userids[$current_userid] = $current_userid;
 	$user_groupids = [];
@@ -346,9 +361,6 @@ if (hasRequest('form')) {
 	$data['current_user_userid'] = $current_userid;
 	$data['form_refresh'] = getRequest('form_refresh');
 
-	// config
-	$data['config'] = select_config();
-
 	// advanced labels
 	$data['labelTypes'] = sysmapElementLabel();
 	$data['labelTypesLimited'] = $data['labelTypes'];
@@ -371,9 +383,7 @@ if (hasRequest('form')) {
 	order_result($data['iconMaps'], 'name');
 
 	// render view
-	$mapView = new CView('monitoring.sysmap.edit', $data);
-	$mapView->render();
-	$mapView->show();
+	echo (new CView('monitoring.sysmap.edit', $data))->getOutput();
 }
 else {
 	CProfile::delete('web.maps.sysmapid');
@@ -393,8 +403,6 @@ else {
 		DBend();
 	}
 
-	$config = select_config();
-
 	$data = [
 		'filter' => [
 			'name' => CProfile::get('web.sysmapconf.filter_name', '')
@@ -402,14 +410,16 @@ else {
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
 		'profileIdx' => 'web.sysmapconf.filter',
-		'active_tab' => CProfile::get('web.sysmapconf.filter.active', 1)
+		'active_tab' => CProfile::get('web.sysmapconf.filter.active', 1),
+		'allowed_edit' => $allowed_edit
 	];
 
 	// get maps
+	$limit = CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1;
 	$data['maps'] = API::Map()->get([
 		'output' => ['sysmapid', 'name', 'width', 'height'],
 		'sortfield' => $sortField,
-		'limit' => $config['search_limit'] + 1,
+		'limit' => $limit,
 		'search' => [
 			'name' => ($data['filter']['name'] === '') ? null : $data['filter']['name']
 		],
@@ -418,8 +428,20 @@ else {
 
 	order_result($data['maps'], $sortField, $sortOrder);
 
-	// paging
-	$data['paging'] = getPagingLine($data['maps'], $sortOrder, new CUrl('sysmaps.php'));
+	// pager
+	if (hasRequest('page')) {
+		$data['page'] = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$data['page'] = 1;
+	}
+	else {
+		$data['page'] = CPagerHelper::loadPage($page['file']);
+	}
+
+	CPagerHelper::savePage($page['file'], $data['page']);
+
+	$data['paging'] = CPagerHelper::paginate($data['page'], $data['maps'], $sortOrder, new CUrl('sysmaps.php'));
 
 	if (CWebUser::getType() != USER_TYPE_SUPER_ADMIN) {
 		$editable_maps = API::Map()->get([
@@ -436,9 +458,7 @@ else {
 	}
 
 	// render view
-	$mapView = new CView('monitoring.sysmap.list', $data);
-	$mapView->render();
-	$mapView->show();
+	echo (new CView('monitoring.sysmap.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
